LFRtools = {};
LFRtools.frame = CreateFrame("Frame");
LFRtools.frame:RegisterEvent("ADDON_LOADED");
LFRtools.frame:RegisterEvent("INSPECT_READY");
LFRtools.frame:RegisterEvent("PLAYER_ALIVE");
LFRtools.frame:RegisterEvent("READY_CHECK");
--Group scan
LFRtools.frame:RegisterEvent("RAID_INSTANCE_WELCOME");
LFRtools.frame:RegisterEvent("GROUP_JOINED");
LFRtools.frame:RegisterEvent("PARTY_LEADER_CHANGED");
--Member scan
LFRtools.frame:RegisterEvent("UNIT_CONNECTION");
LFRtools.frame:RegisterEvent("UNIT_NAME_UPDATE");
--LFRtools.frame:RegisterEvent("PARTY_MEMBER_ENABLE");
LFRtools.frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
LFRtools.frame:RegisterEvent("ROLE_CHANGED_INFORM");

LFRtools.queue = {};
LFRtools.check = {};
--LFRtools.scanned = {};

function LFRtools.onEvent(frame, event, arg, ...)
	if (event == "INSPECT_READY") then
		local _, _, _, _, _, name, realm = GetPlayerInfoByGUID(arg);
		if (realm) and (realm ~= "") then
			name = name .. "-" .. realm;
		end
		for i, entry in ipairs(LFRtools.queue) do
			if (name == entry) then
				table.remove(LFRtools.queue, i);
				LFRtools.inspect(entry);
				ClearInspectPlayer();
				LFRtools.processQueue();
				return;
			end
		end
	elseif (event == "ROLE_CHANGED_INFORM") then
		local officer = ...;
		if (not UnitIsUnit(officer, "player")) then
			LFRtools.enqueue(arg);
		end
	elseif (event == "UNIT_CONNECTION") then
		local connected = ...;
		if (connected) then
			LFRtools.enqueue(arg);
		end
	--elseif (event == "PARTY_MEMBER_ENABLE") then
	--	LFRtools.enqueue(arg);
	elseif (event == "UNIT_NAME_UPDATE") then
		--if (UnitInParty(arg) or UnitInRaid(arg)) then
			LFRtools.enqueue(arg);
		--end
	elseif (event == "PLAYER_SPECIALIZATION_CHANGED") then
		if (arg ~= nil) then
			LFRtools.enqueue(arg);
		end
	elseif (event == "GROUP_JOINED") or (event == "RAID_INSTANCE_WELCOME") then
		LFRtools.check = {};
		LFRtools.scanGroup();
	elseif (event == "PARTY_LEADER_CHANGED") then
		if (UnitIsGroupLeader("player")) then
			LFRtools.scanGroup();
		end
	elseif (event == "READY_CHECK") then
		local chatType = IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and "INSTANCE_CHAT" or (IsInRaid() and "RAID" or "PARTY");
		local msg = "", raidRole;
		for i, entry in ipairs(LFRtools.check) do
			if (UnitInParty(entry) or UnitInRaid(entry)) then
				raidRole = UnitGroupRolesAssigned(entry);
				if (entry ~= "NONE") then
					msg = msg .. LFRtools.unitName(entry) .. " (" .. LFRtools.roleName(raidRole) .. "), "
				end
			end
		end
		if (msg ~= "") then
			SendChatMessage("LFRtools: " .. msg .. "has/have an incorrect specialization!", chatType);
		end
	elseif (event == "PLAYER_ALIVE") then
		if (#LFRtools.queue > 0) then
			--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue resumed");
			LFRtools.processQueue();
		end
	elseif (event == "ADDON_LOADED") and (arg == "LFRtools") then
		LFRtools.frame:UnregisterEvent("ADDON_LOADED");
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Addon loaded");
		if (IsInGroup()) then
			LFRtools.scanGroup();
		end
	end
end
LFRtools.frame:SetScript("OnEvent", LFRtools.onEvent);

function LFRtools.inspect(unit)
	local raidRole = UnitGroupRolesAssigned(unit);
	local _, _, _, _, _, specRole = GetSpecializationInfoByID(GetInspectSpecialization(unit));
	local chatType = IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and "INSTANCE_CHAT" or (IsInRaid() and "RAID" or "PARTY");
	if (specRole == nil) then --Something went wrong.
		table.insert(LFRtools.queue, unit);
		return;
	end
	if (IsInGroup(LE_PARTY_CATEGORY_HOME)) and (not InCombatLockdown()) and ((UnitIsGroupLeader("player")) or (UnitIsRaidOfficer("player"))) then
		UnitSetRole(unit, specRole);
		return;
	end
	if (raidRole == "NONE") then
		return;
	elseif (raidRole ~= specRole) then
		SendChatMessage("LFRtools: " .. LFRtools.unitName(unit) .. " (" .. LFRtools.roleName(raidRole) .. ") is in " .. LFRtools.roleName(specRole) .. " specialization!", chatType);
		for i, entry in ipairs(LFRtools.check) do
			if (unit == entry) then
				return;
			end
		end
		table.insert(LFRtools.check, unit);
	else
		for i, entry in ipairs(LFRtools.check) do
			if (unit == entry) then
				local sex = UnitSex(unit);
				if (sex == 2) then
					sex = "his";
				elseif (sex == 3) then
					sex = "her";
				else
					sex = "its";
				end
				SendChatMessage("LFRtools: " .. LFRtools.unitName(unit) .. " (" .. LFRtools.roleName(raidRole) .. ") has corrected " .. sex .. " specialization.", chatType);
				table.remove(LFRtools.check, i);
				return;
			end
		end
	end
end

function LFRtools.scanGroup()
	local unitType = IsInRaid() and "raid" or "party";
	for i = 1, GetNumGroupMembers() do
		LFRtools.enqueue(unitType .. i);
	end
	--[[
	if (#LFRtools.queue > 0) then
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queued " .. #LFRtools.queue);
	end
	--]]
end

function LFRtools.enqueue(unit)
	local canBeTank, canBeHealer, canBeDPS = UnitGetAvailableRoles(unit);
	if (not canBeTank) and (not canBeHealer) and (canBeDPS) then
		if (IsInGroup(LE_PARTY_CATEGORY_HOME)) and (not InCombatLockdown()) and ((UnitIsGroupLeader("player")) or (UnitIsRaidOfficer("player"))) then
			UnitSetRole(unit, "DAMAGER");
		end
		return;
	end
	local name, realm = UnitName(unit);
	if (name == nil) or (name == "Unknown") then
		return;
	elseif (realm) and (realm ~= "") then
		name = name .. "-" .. realm;
	end
	for i, entry in ipairs(LFRtools.queue) do
		if (name == entry) then
			return;
		end
	end
	table.insert(LFRtools.queue, name);
	if (not UnitIsDead("player")) and (not LFRtools.frame:GetScript("OnUpdate")) then
		LFRtools.processQueue();
	end
	--[[
	for i, entry in ipairs(LFRtools.scanned) do
		if (name == entry) then
			return;
		end
	end
	table.insert(LFRtools.scanned, name);
	]]--
end

function LFRtools.unitName(unit)
	local name, realm = UnitName(unit);
	if (not realm) or (realm == "") then
		realm = GetRealmName();
	end
	return name .. "-" .. realm;
end

function LFRtools.roleName(role)
	return role == "DAMAGER" and "DPS" or role;
end

function LFRtools.onUpdate(frame, elapsed)
	LFRtools.total = LFRtools.total + elapsed;
	if (LFRtools.total >= 5) then --Timeout
		table.insert(LFRtools.queue, table.remove(LFRtools.queue, 1)); --Shuffle
		LFRtools.processQueue();
	end
end

function LFRtools.processQueue()
	if (table.getn(LFRtools.queue) ~= 0) then
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools-Debug: Queued " .. #LFRtools.queue);
		if (UnitIsDead("player")) then
			LFRtools.frame:SetScript("OnUpdate", nil);
			--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue paused");
			return;
		end
		for i, entry in ipairs(LFRtools.queue) do
			if (UnitIsConnected(entry)) and ((UnitInParty(entry)) or (UnitInRaid(entry))) then
				if (CanInspect(entry)) then
					LFRtools.total = 0; --Reset timeout
					LFRtools.frame:SetScript("OnUpdate", LFRtools.onUpdate);
					NotifyInspect(entry);
					return;
				end
			else
				table.remove(LFRtools.queue, i);
				LFRtools.processQueue();
				return;
			end
		end
		LFRtools.total = 0; --Reset timeout
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: None of queued in range");
	else
		LFRtools.frame:SetScript("OnUpdate", nil);
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue done");
	end
end
